/*
 * Copyright (c) 2004, University of Illinois,Distributed Systems Group 
 * Large & Dynamic Groups Size Estimation project members:
 * Dimitrios Psaltoulis
 * Dionysios Kostoulas
 * Indranil Gupta
 * Al Demers
 * Ken Birman
 *  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * - Neither the name of the University of Illinois, Urbana-Champaign
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * GossipMessage.java
 *
 * Created on June 17, 2004, 12:51 PM
 */

package SizeEstimationAlgorithm;

import java.net.*;
import java.io.*;

import SizeEstimationAlgorithm.IntervalCounting.utils.*;
/**
 *
 * @author  dimitrios psaltoulis, dionysios kostoulas
 */

// gossip message class, currently used by both Interval Density and Hop Counting algorithm
// hopDistace and report used by Hop Counting
// peerList and interval used by Interval Density
// possible implementation of a class hierarchy needed for GossipMessage

public class GossipMessage implements Serializable {
    public GossipMessage(GossipId gId, int hDist, boolean rep){
        gossipId = gId;
        hopDistance = hDist;
        report = rep;
    }
    
    public GossipMessage(GossipId gId, MemberList pList, Interval intv) {
        gossipId = gId;
        peerList = pList;
        interval = intv;
    }
    
    public GossipId getGossipId() { return gossipId; }
    public MemberList getPeerList() { return peerList; }
    public void setPeerList(MemberList l) { peerList = l; }
    public Interval getInterval() { return interval; }
    public void setInterval(Interval i) { interval = i; }
    public int getHopDistance() { return hopDistance; }
    public void incHopDistance() { hopDistance++; }
    public boolean getReport() { return report; }
    public void setReport(boolean r) { report=r; }
    
    private GossipId gossipId;
    private int hopDistance;
    private boolean report = false;
    private MemberList peerList;
    private Interval interval;
}
