/*
 * Copyright (c) 2004, University of Illinois,Distributed Systems Group 
 * Large & Dynamic Groups Size Estimation project members:
 * Dimitrios Psaltoulis
 * Dionysios Kostoulas
 * Indranil Gupta
 * Al Demers
 * Ken Birman
 *  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * - Neither the name of the University of Illinois, Urbana-Champaign
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * MyList.java
 *
 * Created on June 17, 2004, 1:07 PM
 */

package SizeEstimationAlgorithm.IntervalCounting.utils;

import java.io.*;
/**
 *
 * @author  dionysios kostoulas
 */
// class of list of peers with hashIDs within interval

public class MemberList implements Serializable{
    public MemberList() {
        size = 0;
    }
    
    public final int maxmemory = 100;
    public final int minmemory = 5;
    
    public ListEntry[] getEntries() { return entries; }
    public int getSize() { return size; }
    
    public int contains(HashId aId) {
        for (int i = 0; i < size; i++) {
            if (entries[i].getPid().equals(aId)) return i;
        }
        
        return -1;
    }
    
    public void add(ListEntry aEntry) {
        entries[size] = aEntry;
        size++;
    }
    
    public void delete(int index) {
        for (int i = index; i < entries.length - 1; i++) {
            entries[i] = entries[i+1];
        }
        
        size--;
    }
    
    public void mergeWith(MemberList aList) {
        for (int i = 0; i < aList.getSize(); i++) {
            ListEntry current = aList.entries[i];
            int pos = contains(current.getPid());
            if (pos == -1) add(current);
            else if (current.isFresherThan(entries[pos])) {
                entries[pos].update(current.getHeartbeat(), current.getLastModified());
            }
        }
    }
    
    public void clear(int rounds) {
        for (int i = 0; i < size; i++) {
            if (entries[i].isRedundant(rounds)) {
                delete(i);
            }
        }
    }
    
    public ListEntry[] entries = new ListEntry[500];
    private int size;
}