/*
 * Copyright (c) 2004, University of Illinois,Distributed Systems Group
 * Large & Dynamic Groups Size Estimation project members:
 * Dimitrios Psaltoulis
 * Dionysios Kostoulas
 * Indranil Gupta
 * Al Demers
 * Ken Birman
 *  All rights reserved.
 *
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * - Neither the name of the University of Illinois, Urbana-Champaign
 * nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * ConnectionHandler.java
 *
 * Created on June 17, 2004, 4:08 PM
 */

package SizeEstimationAlgorithm;

import java.net.*;
import java.io.*;

import SizeEstimationAlgorithm.HopCounting.*;
import SizeEstimationAlgorithm.IntervalCounting.*;

/**
 *
 * @author  dimitrios psaltoulis
 */
public class Server extends Thread {
    public Server(int port, Object o) {
        serverPort = port;
        if (o instanceof HopCountingAlgorithm)
            alg = (HopCountingAlgorithm)o;
        else if (o instanceof IntervalCountingAlgorithm)
            alg = (IntervalCountingAlgorithm)o;
        /*if (GossipAlgorithm.DEBUG)*/ System.out.println("Server thread created\n");
    }
    
    public void run() {
        try {
            /*if (GossipAlgorithm.DEBUG)*/ System.out.println("Server IP: " + InetAddress.getLocalHost());
            /*if (GossipAlgorithm.DEBUG)*/ System.out.println("Server port: " + serverPort + "\n");
            byte buffer[] = new byte[64*1024];
            DatagramPacket incoming = new DatagramPacket(buffer, buffer.length);
            datagramSocket = new DatagramSocket(serverPort);
            while(true) {
                datagramSocket.receive(incoming);
                buffer = incoming.getData();
                ByteArrayInputStream byteStream = new ByteArrayInputStream(buffer);
                ObjectInputStream is = new ObjectInputStream(new BufferedInputStream(byteStream));
                try {
                    Object o = is.readObject();
                    
                    is.close();
                    
                    if (GossipAlgorithm.DEBUG) System.out.println("\nMessage received from: "+
                    incoming.getAddress().getCanonicalHostName());
                    alg.receiveMessage((GossipMessage)o);
                }
                catch (ClassNotFoundException e){
                    System.err.println(e);
                }
                catch (ClassCastException e) {
                    System.err.println(e);
                }
            }
        }
        catch (IOException e) { /* write error message to output and notify gui that you are exiting */ }
    }
    
    public int serverPort;
    public DatagramSocket datagramSocket;
    
    public GossipAlgorithm alg;
}
